"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPushNotification = void 0;
const package_name_1 = require("./package-name");
const contains_1 = require("./contains");
/**
 * Gets push notification from deep
 *
 * @throws {Error} If title is not found
 * @throws {Error} If body is not found
 */
function getPushNotification({ deep, pushNotificationLinkId, }) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const titleTypeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.PushNotificationTitle]);
        const bodyTypeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.PushNotificationBody]);
        const imageUrlTypeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.PushNotificationImageUrl]);
        const iconUrlTypeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.PushNotificationIconUrl]);
        const pushNotificationTreeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.PushNotificationTree]);
        const { data: linksDownToParentPushNotificationMp } = yield deep.select({
            up: {
                parent_id: { _eq: pushNotificationLinkId },
                tree_id: { _eq: pushNotificationTreeLinkId },
            },
        }, {
            returning: `${deep.selectReturning}
    to {
      ${deep.selectReturning}
    }
    `,
        });
        const linkWithTitle = linksDownToParentPushNotificationMp.find((link) => link.type_id === titleTypeLinkId);
        if (!linkWithTitle) {
            throw new Error(`A link with type ##${titleTypeLinkId} is not found`);
        }
        const linkWithBody = linksDownToParentPushNotificationMp.find((link) => link.type_id === bodyTypeLinkId);
        if (!linkWithBody) {
            throw new Error(`A link with type ##${bodyTypeLinkId} is not found`);
        }
        const linkWithImageUrl = linksDownToParentPushNotificationMp.find((link) => link.type_id === imageUrlTypeLinkId);
        const linkWithIconUrl = linksDownToParentPushNotificationMp.find((link) => link.type_id === iconUrlTypeLinkId);
        return {
            title: linkWithTitle === null || linkWithTitle === void 0 ? void 0 : linkWithTitle.to.value.value,
            body: linkWithBody.to.value.value,
            image: (_a = linkWithImageUrl === null || linkWithImageUrl === void 0 ? void 0 : linkWithImageUrl.to.value) === null || _a === void 0 ? void 0 : _a.value,
            icon: (_b = linkWithIconUrl === null || linkWithIconUrl === void 0 ? void 0 : linkWithIconUrl.to.value) === null || _b === void 0 ? void 0 : _b.value,
        };
    });
}
exports.getPushNotification = getPushNotification;
