"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerDevice = void 0;
const device_1 = require("@capacitor/device");
const push_notifications_1 = require("@capacitor/push-notifications");
const messaging_1 = require("@firebase/messaging");
const insert_device_registration_token_1 = require("./insert-device-registration-token");
const package_name_1 = require("./package-name");
const contains_1 = require("./contains");
/**
 * Registers device
 */
function registerDevice({ deep, deviceLinkId, firebaseMessaging, callback, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const { platform } = yield device_1.Device.getInfo();
        if (platform === 'web') {
            const serviceWorkerRegistration = yield navigator.serviceWorker.register('./firebase-messaging-sw.js', { scope: 'firebase-cloud-messaging-push-scope' });
            const webPushCertificateLink = yield getPushCertificateLink({ deep });
            const webPushCertificate = webPushCertificateLink.value.value;
            const deviceRegistrationToken = yield (0, messaging_1.getToken)(firebaseMessaging, {
                serviceWorkerRegistration,
                vapidKey: webPushCertificate,
            });
            onDeviceRegistration({ value: deviceRegistrationToken });
            (0, messaging_1.onMessage)(firebaseMessaging, (payload) => __awaiter(this, void 0, void 0, function* () {
                console.log('[firebase-messaging-sw.js] Received background message ', payload);
                if (!payload.notification) {
                    return;
                }
                if (!payload.notification.title) {
                    return;
                }
                // Let's check if the browser supports notifications
                if (!('Notification' in window)) {
                    throw new Error('This browser does not support desktop notification');
                }
                // Let's check whether notification permissions have already been granted
                else if (Notification.permission === 'granted') {
                    // If it's okay, let's create a notification
                    new Notification(payload.notification.title, payload.notification);
                }
                // Otherwise, we need to ask the user for permission
                else if (Notification.permission !== 'denied') {
                    const permission = yield Notification.requestPermission();
                    if (permission === 'granted') {
                        new Notification(payload.notification.title, payload.notification);
                    }
                }
            }));
        }
        else {
            yield push_notifications_1.PushNotifications.removeAllListeners();
            yield push_notifications_1.PushNotifications.addListener('registration', onDeviceRegistration);
            yield push_notifications_1.PushNotifications.register();
        }
        function onDeviceRegistration({ value: deviceRegistrationToken, }) {
            return __awaiter(this, void 0, void 0, function* () {
                if (callback) {
                    callback({ deviceRegistrationToken });
                }
                else {
                    yield (0, insert_device_registration_token_1.insertDeviceRegistrationToken)({
                        deep,
                        deviceRegistrationToken,
                        deviceLinkId,
                    });
                }
            });
        }
    });
}
exports.registerDevice = registerDevice;
function getPushCertificateLink({ deep }) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const webPushCertificateTypeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.WebPushCertificate]);
        const usesWebPushCertificateTypeLinkId = yield deep.id(package_name_1.FIREBASE_PUSH_NOTIFICATION_PACKAGE_NAME, contains_1.FirebasePushNotificationContains[contains_1.FirebasePushNotificationContains.UsesWebPushCertificate]);
        const containTypeLinkId = yield deep.id('@deep-foundation/core', 'Contain');
        const selectData = {
            _or: [
                {
                    type_id: webPushCertificateTypeLinkId,
                    in: {
                        type_id: containTypeLinkId,
                        from_id: deep.linkId,
                    },
                },
                {
                    type_id: usesWebPushCertificateTypeLinkId,
                    from_id: deep.linkId,
                },
            ],
        };
        const { data } = yield deep.select(selectData);
        let webPushCertificateLink;
        if (data.length === 0) {
            throw new Error(`Select with data ${JSON.stringify(selectData)} returned empty result`);
        }
        const usesWebPushCertificateLinks = data.filter((link) => link.type_id === usesWebPushCertificateTypeLinkId);
        if (usesWebPushCertificateLinks.length > 1) {
            throw new Error(`Select with data ${JSON.stringify(selectData)} returned more than one link of type ##${usesWebPushCertificateTypeLinkId}: ${usesWebPushCertificateLinks
                .map((link) => `##${link.id}`)
                .join(',')}`);
        }
        else if (usesWebPushCertificateLinks.length === 1) {
            const usesWebPushCertificateLink = usesWebPushCertificateLinks[0];
            webPushCertificateLink = data.find((link) => link.id === usesWebPushCertificateLink.to_id);
        }
        else {
            const webPushCertificateLinks = data.filter((link) => link.type_id === webPushCertificateTypeLinkId);
            if (webPushCertificateLinks.length > 1) {
                throw new Error(`Select with data ${JSON.stringify(selectData)} returned more than one link of type ##${webPushCertificateTypeLinkId}: ${webPushCertificateLinks
                    .map((link) => `##${link.id}`)
                    .join(',')}`);
            }
            webPushCertificateLink = webPushCertificateLinks[0];
        }
        if (!webPushCertificateLink) {
            throw new Error(
            // `A link with type ##${usesWebPushCertificateTypeLinkId}, from ##${deep.linkId} to ##${webPushCertificateTypeLinkId} is not found`
            `${JSON.stringify(selectData)} query has found 0 links`);
        }
        if (!((_a = webPushCertificateLink.value) === null || _a === void 0 ? void 0 : _a.value)) {
            throw new Error(`##${webPushCertificateLink.id} must have a value`);
        }
        return webPushCertificateLink;
    });
}
